@@ -14,16 +14,17 @@ import json
 import warnings
 
 from django.db.models import fields as dj_fields
-from django.core import exceptions as dj_exceptions
 from django.utils.encoding import force_unicode
 
 from m3 import actions as m3_actions
-from m3.actions.interfaces import IMultiSelectablePack
+from m3.actions.interfaces import ISelectablePack
+from m3.actions import DeclarativeActionContext as _DAC
 from m3 import RelatedError, ApplicationLogicException
-from m3.db import safe_delete
+from m3.db import safe_delete, tools as dbtools
 from m3_ext.ui.fields.complex import ExtSearchField
 from m3_ext.ui import results as ui_results
 
+
 import ui
 import tools
 import exceptions
@@ -67,7 +68,6 @@ class BaseAction(m3_actions.Action):
 
     def get_perm_code(self, subpermission=None):
         """
-        Возвращает код права
 
         :param subpermission: Код подправа доступа
         :type subpermission: str
@@ -127,6 +127,15 @@ class BaseWindowAction(BaseAction):
     """
     Базовый Action показа окна
     """
+
+    suffixes = ('ui',)
+
+    def build_context(self, request, suffix):
+        if suffix == 'ui':
+            return _DAC()
+        else:
+            return super(BaseWindowAction, self).build_context(request, suffix)
+
     def create_window(self):
         """
         Метод инстанцирует окно и помещает экземпляр в атрибут self.win
@@ -197,7 +206,23 @@ class BaseWindowAction(BaseAction):
         """
         pass
 
-    def run(self, request, context):
+    @property
+    def ui(self):
+        return self.get_absolute_url() + ":ui"
+
+    def get_model(self, request, context):
+        """
+        Возвращает сериализованную viewmodel,
+        которая должна отображаться в окне
+
+        :param request: Request
+        :type request: django.http.HttpRequest
+        :param context: Context
+        :type context: m3.actions.context.DeclarativeActionContext
+        """
+        return {}
+
+    def run(self, request, context, suffix=None):
         """
         Тело Action, вызывается при обработке запроса к серверу.
 
@@ -210,18 +235,25 @@ class BaseWindowAction(BaseAction):
 
            Обычно не требует перекрытия
         """
-        new_self = copy.copy(self)
-        new_self.win_params = (
-            getattr(self.__class__, 'win_params', None) or {}
-        ).copy()
-        new_self.request = request
-        new_self.context = context
-        new_self.set_window_params()
-        new_self.create_window()
-        new_self._apply_window_params()
-        new_self.configure_window()
-        return ui_results.ExtUIScriptResult(
-            new_self.win, context=new_self.context)
+        if suffix == 'ui':
+            new_self = copy.copy(self)
+            new_self.win_params = (
+                getattr(self.__class__, 'win_params', None) or {}
+            ).copy()
+            new_self.request = request
+            new_self.context = context
+            new_self.set_window_params()
+            new_self.create_window()
+            new_self._apply_window_params()
+            new_self.configure_window()
+            result = ui_results.UIResult(new_self.win)
+        else:
+            result = ui_results.DataResult(
+                model=self.get_model(request, context),
+                ui=self.ui,
+                context=context
+            )
+        return result
 
 
 #==============================================================================
@@ -232,7 +264,14 @@ class ObjectListWindowAction(BaseWindowAction):
     Базовый Action показа окна списка объектов
     """
     is_select_mode = False
-    """Режим показа окна (True - выбор, False - список)"""
+    """Режим показа окна
+    (True - выбор одного значения,
+    False - список)"""
+
+    is_multi_select_mode = False
+    """Режим показа окна
+    (True - выбор нескольких значений,
+    False - список)"""
 
     perm_code = 'view'
     """Код доступа"""
@@ -240,6 +279,7 @@ class ObjectListWindowAction(BaseWindowAction):
     def set_window_params(self):
         params = self.win_params
         params['is_select_mode'] = self.is_select_mode
+        params['multi_select'] = self.is_multi_select_mode
         params['pack'] = self.parent
         params['title'] = self.parent.title
         params['height'] = self.parent.height
@@ -280,8 +320,9 @@ class ObjectMultiSelectWindowAction(ObjectSelectWindowAction):
     """
     Базовый Action показа окна списка выбора нескольких объектов из списка
     """
-    def create_window(self):
-        self.win = self.parent.multi_select_window()
+
+    is_multi_select_mode = True
+
 
 #==============================================================================
 # ObjectEditWindowAction
@@ -293,14 +334,7 @@ class ObjectEditWindowAction(BaseWindowAction):
     perm_code = 'edit'
 
     def set_window_params(self):
-        try:
-            obj, create_new = self.parent.get_obj(self.request, self.context)
-        except self.parent.get_not_found_exception():
-            raise ApplicationLogicException(self.parent.MSG_DOESNOTEXISTS)
-
         params = self.win_params.copy()
-        params['object'] = obj
-        params['create_new'] = create_new
         params['form_url'] = self.parent.save_action.get_absolute_url()
 
         read_only = getattr(self.parent, 'read_only', None) or (
@@ -309,7 +343,7 @@ class ObjectEditWindowAction(BaseWindowAction):
         params['read_only'] = read_only
         params['title'] = self.parent.format_window_title(
             u'Просмотр' if read_only else
-            u'Добавление' if create_new else
+            #u'Добавление' if create_new else
             u'Редактирование'
         )
 
@@ -319,13 +353,19 @@ class ObjectEditWindowAction(BaseWindowAction):
                 params, self.request, self.context))
 
     def create_window(self):
-        assert 'create_new' in self.win_params, (
-            u'You must call "set_window_params" method of superclass!')
         self.win = self.handle(
             'create_window',
             self.parent.create_edit_window(
-                self.win_params['create_new'], self.request, self.context))
+                create_new=False,
+                request=self.request,
+                context=self.context))
 
+    def get_model(self, request, context):
+        # модель редактирования предоставляет пак
+        return self.handle(
+            'get_editing_model',
+            self.parent.get_editing_model(request, context)
+        )
 
 #==============================================================================
 # ObjectAddWindowAction
@@ -333,15 +373,27 @@ class ObjectEditWindowAction(BaseWindowAction):
 class ObjectAddWindowAction(ObjectEditWindowAction):
     """
     Базовый Action показа окна добавления объекта.
-
-    .. note::
-        Отдельный action для уникальности short_name
     """
-
     perm_code = 'add'
 
-    pass
+    def build_context(self, *args, **kwargs):
+        ctx = super(ObjectAddWindowAction, self).build_context(*args, **kwargs)
+        id_param = self.parent.id_param_name
+        assert not hasattr(ctx, id_param), (
+            'Экшн создания нового объекта не должен получать id param '
+            'своего пака!'
+        )
+        # т.к. создается новый объект, id param должен быть "пустым"
+        setattr(ctx, id_param, None)
+        return ctx
 
+    def create_window(self):
+        self.win = self.handle(
+            'create_window',
+            self.parent.create_edit_window(
+                create_new=True,
+                request=self.request,
+                context=self.context))
 
 #==============================================================================
 # ObjectSaveAction
@@ -559,10 +611,7 @@ class ObjectRowsAction(BaseAction):
             col, subcol = (col.split('.', 1) + [None])[:2]
             # ------- если есть подиндекс - идем вглубь
             if subcol:
-                try:
-                    obj = getattr(obj, col, None)
-                except dj_exceptions.ObjectDoesNotExist:
-                    obj = None
+                obj = getattr(obj, col, None)
                 sub_dict = result.setdefault(col, {})
                 parse_data_indexes(obj, subcol, sub_dict)
             else:
@@ -575,17 +624,12 @@ class ObjectRowsAction(BaseAction):
                 ):
                     fld = None
                 # получаем значение
-                try:
-                    obj = getattr(obj, col, None)
-                except dj_exceptions.ObjectDoesNotExist:
-                    obj = None
+                obj = getattr(obj, col, None)
                 if fld:
                     try:
                         obj = obj.display()
                     except AttributeError:
-                        # аттрибут choises существует у Field
-                        # но отсутствует у RelatedObject
-                        if hasattr(fld, 'choices') and fld.choices:
+                        if fld.choices:
                             # если получаемый атрибут - поле, имеющее choices
                             # пробуем найти соответствующий значению вариант
                             for ch in fld.choices:
@@ -825,7 +869,7 @@ class BasePack(m3_actions.ActionPack):
 #==============================================================================
 # ObjectPack
 #==============================================================================
-class ObjectPack(BasePack, IMultiSelectablePack):
+class ObjectPack(BasePack, ISelectablePack):
     """
     Пак с экшенам, реализующими специфичную для работы с моделью действиями по
     добавлению, редактированию, удалению (CRUD actions)
@@ -927,6 +971,15 @@ class ObjectPack(BasePack, IMultiSelectablePack):
     Класс django-модели, для которой будет формироваться справочник
     """
 
+    serialization_rules = ([], [])
+    """
+    Правила (де)сериализации объектов модели для окон редактирования
+
+    .. note::
+
+        См. параметры include и exclude в (m3.db.tools.matcher)
+    """
+
     columns = [
         {
             'header': u'Наименование',
@@ -972,22 +1025,10 @@ class ObjectPack(BasePack, IMultiSelectablePack):
                'width':200,
                'header':u'Родитель',
                'renderer':'parent_render'
-               'select_related': ('parent',),
             },
         ]
     """
 
-    select_related = None
-    """
-    Перечень related полей, для передачи в Manager.select_related.
-
-    Если None и в columns не будет найдено related полей, то в rows_action
-    select_related не будет выполняться.
-
-    Если любая пустая коллекция, то вызовется select_related без параметров,
-    что может сказаться на производительности
-    """
-
     filter_engine_clz = filters.MenuFilterEngine
     """
     Класс конструктора фильтра для грида
@@ -1076,17 +1117,11 @@ class ObjectPack(BasePack, IMultiSelectablePack):
     Класс отвечающий за отображение окна со списком объектов
     """
 
-    select_window = ui.BaseSelectWindow  # Форма выбора
+    select_window = ui.BaseSelectWindow  # Форма выбора @UndefinedVariable
     """
     Класс отвечающий за отображение окна выбора из списка объектов
     """
 
-    multi_select_window = ui.BaseMultiSelectWindow  # Форма выбора
-    """
-    Класс отвечающий за отображение окна множественного выбора
-    из списка объектов
-    """
-
     width, height = 600, 400
     """
     Размеры окна по умолчанию
@@ -1137,8 +1172,9 @@ class ObjectPack(BasePack, IMultiSelectablePack):
         self.list_window_action = ObjectListWindowAction()
         #: Экшен показа окна со списком для выбора объектов
         self.select_window_action = ObjectSelectWindowAction()
-        #: Экшен с получения данных объектов / редактирование строк
+         #: Экшен показа окна со списком для выбора объектов
         self.multi_select_window_action = ObjectMultiSelectWindowAction()
+
         #: Экшен с получения данных объектов / редактирование строк
         self.rows_action = ObjectRowsAction()
         # Но привязать их все равно нужно
@@ -1183,7 +1219,6 @@ class ObjectPack(BasePack, IMultiSelectablePack):
         self._columns_flat = []
         self._all_search_fields = (self.search_fields or [])[:]
         self._sort_fields = {}
-        self._select_related_fields = (self.select_related or [])[:]
 
         def flatify(cols):
             for c in cols:
@@ -1199,7 +1234,7 @@ class ObjectPack(BasePack, IMultiSelectablePack):
                         sort_fields = c.get('sort_fields', [field])
                         try:
                             sort_fields = list(sort_fields)
-                        except TypeError:
+                        except:
                             sort_fields = [sort_fields]
                         self._sort_fields[data_index] = sort_fields
                     # поле для фильтрации
@@ -1207,22 +1242,9 @@ class ObjectPack(BasePack, IMultiSelectablePack):
                         search_fields = c.get('search_fields', [field])
                         try:
                             search_fields = list(search_fields)
-                        except TypeError:
+                        except:
                             search_fields = [search_fields]
                         self._all_search_fields.extend(search_fields)
-                    # related fields
-                    related_fields = c.get("select_related")
-                    if not related_fields:
-                        lookup = "__".join(tools.get_related_fields(
-                            self.model, data_index.split(".")))
-                        related_fields = lookup and [lookup] or []
-                    try:
-                        related_fields = list(related_fields)
-                    except TypeError:
-                        related_fields = [related_fields]
-                    for f in related_fields:
-                        if f not in self._select_related_fields:
-                            self._select_related_fields.append(f)
         flatify(self.columns)
 
         # подключение механизма фильтрации
@@ -1257,14 +1279,13 @@ class ObjectPack(BasePack, IMultiSelectablePack):
         :rtype: dict
         """
         result = super(ObjectPack, self).declare_context(action)
-        if action in (
-            self.edit_window_action,
-            self.new_window_action,
-            self.save_action
-        ):
-            result = {self.id_param_name: {'type': tools.int_or_zero}}
+        if action is self.save_action:
+            result = {self.id_param_name: {'type': 'int_or_none',
+                                           'default': None}}
+        elif action is self.edit_window_action:
+            result = {self.id_param_name: {'type': 'int'}}
         elif action is self.delete_action:
-            result = {self.id_param_name: {'type': tools.int_list}}
+            result = {self.id_param_name: {'type': 'int_list'}}
         return result
 
     def get_default_action(self):
@@ -1407,18 +1428,6 @@ class ObjectPack(BasePack, IMultiSelectablePack):
         """
         return self.select_window_action.get_absolute_url()
 
-    def get_multi_select_url(self):
-        """
-        Возвращает адрес формы выбора из списка элементов справочника.
-
-        .. note::
-            Используется для присвоения адресов в прикладном приложении
-
-        :return: url экшена показа окна выбора из списка объектов
-        :rtype: str
-        """
-        return self.multi_select_window_action.get_absolute_url()
-
     def get_edit_url(self):
         """
         Возвращает адрес формы редактирования элемента справочника
@@ -1448,10 +1457,6 @@ class ObjectPack(BasePack, IMultiSelectablePack):
         """
         return self.get_rows_url()
 
-    def get_display_dict(self, key, value_field='id', display_field='name'):
-        print key, value_field, display_field
-        return []
-
     def get_not_found_exception(self):
         """
         Возвращает класс исключения 'объект не найден'
@@ -1512,17 +1517,20 @@ class ObjectPack(BasePack, IMultiSelectablePack):
             ])
         cc.configure_grid(grid)
 
-        #TODO перенести в класс грида, сделать метод add_search_field
         if self.get_search_fields():
             #поиск по гриду если есть по чему искать
-            grid.top_bar.search_field = ExtSearchField(
-                empty_text=u'Поиск', width=200, component_for_search=grid)
-            grid.top_bar.add_fill()
-            grid.top_bar.items.append(grid.top_bar.search_field)
+            search_field = ExtSearchField(
+                empty_text=u'Поиск',
+                width=200,
+                component_item_id=grid.item_id)
+            grid.top_bar.items.append('->')
+            grid.top_bar.items.append(search_field)
 
         grid.row_id_name = self.id_param_name
         grid.allow_paging = self.allow_paging
-        grid.store.remote_sort = self.allow_paging
+        # NR: нынче стора нету у нас, а пагинацию надо бы пробросить
+        if hasattr(grid, 'store'):
+            grid.store.remote_sort = self.allow_paging
 
         self._filter_engine.configure_grid(grid)
 
@@ -1556,6 +1564,22 @@ class ObjectPack(BasePack, IMultiSelectablePack):
         else:
             return self.edit_window()
 
+    def get_editing_model(self, request, context):
+        try:
+            obj, create_new = self.get_obj(request, context)
+        except self.get_not_found_exception():
+            raise ApplicationLogicException(
+                self.MSG_DOESNOTEXISTS)
+        model = self.serialize(obj)
+        return model
+
+    def serialize(self, obj):
+        if hasattr(obj, 'serialize'):
+            # модель может захотеть сериализоваться сама
+            return obj.serialize(*self.serialization_rules)
+        else:
+            return dbtools.model_to_dict(obj, *self.serialization_rules)
+
     def create_list_window(self, is_select_mode, request, context):
         """
         Получить окно списка / выбора объектов
@@ -1607,10 +1631,7 @@ class ObjectPack(BasePack, IMultiSelectablePack):
                 return query.filter(done=True)
 
         """
-        query = self.model.objects.all()
-        if self._select_related_fields or self.select_related is not None:
-            query = query.select_related(*self._select_related_fields)
-        return query
+        return self.model.objects.all().select_related()
 
     def get_search_fields(self, request=None, context=None):
         """
@@ -1785,7 +1806,7 @@ class ObjectPack(BasePack, IMultiSelectablePack):
         :return: Объект модели self.model
         :rtype: django.db.models.Model
         """
-        if row_id == 0:
+        if not row_id:
             record = self.model()
         else:
             record = self.model.objects.get(id=row_id)
@@ -1802,8 +1823,8 @@ class ObjectPack(BasePack, IMultiSelectablePack):
         :return: (Объект модели self.model, create_new)
         :rtype: tuple
         """
-        obj_id = getattr(context, self.id_param_name)
-        create_new = (obj_id == 0)
+        obj_id = getattr(context, self.id_param_name, None)
+        create_new = not obj_id
         record = self.get_row(obj_id)
         return record, create_new
 
@@ -2054,3 +2075,27 @@ def multiline_text_window_result(
             data.replace("\n", r"\n").replace(r"'", r'"'))
         )
     )
+
+#-----------------------------------------------------------------------------
+class MasterDetailWindowAction(BaseWindowAction):
+    """
+    Окно, предоставляющее интерфейс "master-detail"
+    """
+    window_clz = ui.MasterDetailWindow
+
+    @property
+    def master_pack(self):
+        return self.parent
+
+    @property
+    def detail_pack(self):
+        return None
+
+    def create_window(self):
+        self.win = self.window_clz()
+
+    def set_window_params(self):
+        self.win_params.update({
+            'master_pack': self.master_pack,
+            'detail_pack': self.detail_pack,
+        })
