@@ -6,11 +6,15 @@ from m3 import actions as m3_actions
 import objectpack
 from objectpack import tree_object_pack
 from objectpack.filters import FilterByField, ColumnFilterEngine
+from objectpack.demo.controller import obs
 
 import models
 import ui
 
 
+_PF = partial(FilterByField, models.Person)
+
+
 #==============================================================================
 # PersonObjectPack
 #==============================================================================
@@ -23,14 +27,21 @@ class PersonObjectPack(objectpack.ObjectPack):
     add_to_desktop = True
     add_to_menu = True
 
-    edit_window = add_window = objectpack.ui.ModelEditWindow.fabricate(model)
+    edit_window = add_window = objectpack.ui.ModelEditWindow.fabricate(
+        model, model_register=obs
+    )
 
     columns = [
+        {
+            'data_index': '__unicode__',
+            'hidden': True
+        },
         {
             'data_index': 'fullname',
             'header': u'ФИО',
             'sortable': True,
             'sort_fields': ('name', 'surname'),
+
             'filter': {
                 'type': 'string',
                 'custom_field': ('name', 'surname')
@@ -38,6 +49,7 @@ class PersonObjectPack(objectpack.ObjectPack):
         },
         {
             'data_index': 'date_of_birth',
+            "searchable": True,
             'header': u'Дата рождения',
             'filter': {
                 'type': 'date'
@@ -54,6 +66,25 @@ class PersonObjectPack(objectpack.ObjectPack):
     ]
 
 
+class PersonCardObjectPack(PersonObjectPack):
+
+    add_to_desktop = True
+    add_to_menu = True
+
+    model = models.PersonCard
+
+    edit_window = add_window = ui.PersonCardEditWindow
+
+    columns = [
+        {
+            'data_index': 'person.__unicode__',
+            'header': u'ФИО',
+        },
+    ]
+
+    serialization_rules = ([], ['*mother_id', '*father_id'])
+
+
 #==============================================================================
 # CFPersonObjectPack
 #==============================================================================
@@ -71,8 +102,6 @@ class CFPersonObjectPack(objectpack.ObjectPack):
 
     filter_engine_clz = ColumnFilterEngine
 
-    F = partial(FilterByField, model)
-
     columns = [
         {
             'data_index': 'fullname',
@@ -80,21 +109,22 @@ class CFPersonObjectPack(objectpack.ObjectPack):
             'sortable': True,
             'sort_fields': ('name', 'surname'),
             'filter': (
-                F('name') & F('surname')
+                _PF('name', 'name__icontains') &
+                _PF('surname', 'surname__icontains')
             )
         },
         {
             'data_index': 'date_of_birth',
             'header': u'Дата рождения',
             'filter': (
-                F('date_of_birth', '%s__gte', tooltip=u'с') &
-                F('date_of_birth', '%s__lte', tooltip=u'по')
+                _PF('date_of_birth', 'date_of_birth__gte', tooltip=u'с') &
+                _PF('date_of_birth', 'date_of_birth__lte', tooltip=u'по')
             )
         },
         {
             'data_index': 'gender',
             'header': u'Пол',
-            'filter': F('gender')
+            'filter': _PF('gender')
         }
     ]
 
@@ -168,6 +198,21 @@ class TreePack(tree_object_pack.TreeObjectPack):
 #==============================================================================
 # Паки гаражей с инструментом и сотрудницами
 #==============================================================================
+class GarageMDWindowAction(objectpack.actions.MasterDetailWindowAction):
+
+    @property
+    def detail_pack(self):
+        return self.parent._get_model_pack('GarageTool')
+
+    def set_window_params(self):
+        super(GarageMDWindowAction, self).set_window_params()
+        self.win_params.update({
+            'title': self.parent.title,
+            'width': 600,
+            'height': 400,
+        })
+
+
 class GaragePack(objectpack.ObjectPack):
     """
     Гаражи
@@ -180,6 +225,17 @@ class GaragePack(objectpack.ObjectPack):
     add_window = objectpack.ModelEditWindow.fabricate(model)
     edit_window = ui.GarageEditWindow
 
+    def __init__(self):
+        super(GaragePack, self).__init__()
+        self.md_window_action = GarageMDWindowAction()
+        self.actions.append(self.md_window_action)
+
+    def extend_desktop(self, desk):
+        return [
+            desk.Item(self.title, pack=self.list_window_action),
+            desk.Item(self.title + '(MD)', pack=self.md_window_action),
+        ]
+
 
 class ToolPack(objectpack.SlavePack):
     """
@@ -187,6 +243,8 @@ class ToolPack(objectpack.SlavePack):
     """
     model = models.GarageTool
 
+    _is_primary_for_model = True
+
     parents = ['garage']
 
     add_window = edit_window = objectpack.ModelEditWindow.fabricate(
@@ -214,39 +272,11 @@ class StaffPack(objectpack.SlavePack):
         self.actions.append(self.save_staff_action)
 
 
-class ROStaffPack(objectpack.ObjectPack):
-    """
-    Сотрудники гаража для отображения на раб.столе
-    Демонстрирует колоночные фильтры с лукапом вглубь
-    """
-    model = models.GarageStaff
-
-    add_to_menu = add_to_desktop = True
-
-    filter_engine_clz = ColumnFilterEngine
-
-    F = partial(FilterByField, model)
-
-    columns = [
-        {
-            'header': u'ФИО',
-            'data_index': 'person',
-            'filter': (
-                F('person__name') & F('person__surname')
-            )
-        },
-        {
-            'header': u'Гараж',
-            'data_index': 'garage',
-            'filter': F('garage__name')
-        },
-    ]
-
-
 class SelectPersonAction(objectpack.SelectorWindowAction):
     """
     Экшн отображения списка физ.лиц
     """
+
     def configure_action(self, request, context):
         self.callback_url = self.parent.save_staff_action.get_absolute_url()
         self.data_pack = self.parent._get_model_pack('Person')
