@@ -24,6 +24,7 @@ def register_actions():
     """
     controller.action_controller.packs.extend([
         actions.PersonObjectPack(),
+        actions.PersonCardObjectPack(),
         actions.CFPersonObjectPack(),
         actions.BandedColumnPack(),
         actions.TreePack(),
@@ -31,8 +32,6 @@ def register_actions():
         actions.GaragePack(),
         actions.ToolPack(),
         actions.StaffPack(),
-
-        actions.ROStaffPack(),
     ])
 
 
