@@ -4,11 +4,14 @@
 """
 import copy
 from collections import Iterable
+from operator import attrgetter
 
 from itertools import ifilter, ifilterfalse, islice, imap
 
 from django.db.models import query, manager
 
+from m3.db.tools import model_to_dict as _model_to_dict
+
 
 def kwargs_only(*keys):
     keys = set(keys)
@@ -354,12 +357,20 @@ class ModelProxyMeta(type):
                 fields_ = []
                 fields_dict = {}
                 for prefix, meta in [(model.__name__.lower(), meta)] + [
-                        (rel, submeta(meta, rel)) for rel in relations]:
+                    (rel, submeta(meta, rel)) for rel in relations
+                ]:
                     for f in meta.fields:
                         f = add_prefix(f, prefix)
                         fields_.append(f)
                         fields_dict[f.attname] = f
 
+                # сами relations должны быть представлены в виде полей
+                # на верхнем уровне
+                for rel in relations:
+                    f = model._meta.get_field(rel)
+                    fields_.append(copy.copy(f))
+                    fields_dict[rel] = f
+
                 return fields_, fields_dict
 
         # django-подобный класс метаинформации о модели
@@ -535,3 +546,14 @@ class ModelProxy(object):
 
     def safe_delete(self):
         raise NotImplementedError()
+
+    def serialize(self, include=None, exclude=None):
+        if not (include or exclude):
+            # если правила не заданы,
+            # исключаются relations как поля
+            self_name = self.model.__name__.lower()
+            exclude = [
+                ('%s.%s_id' % (self_name, rel))
+                for rel in self.relations
+            ]
+        return _model_to_dict(self, include, exclude)
