@@ -37,18 +37,17 @@ class ObservableMixin(object):
         self._observer = observer
         self._already_registered = set()
 
-    def _invoke(self, request, action, stack):
+    def _invoke(self, request, action, stack, suffix=None):
         """
         Вызов action под наблюдением
         """
-        self._observer.configure()
-
         stack = stack[:]
         self._observer._prepare_for_listening(action, request, stack)
 
         # обработка контроллером
         with action._catcher as catcher:
-            return super(ObservableMixin, self)._invoke(request, action, stack)
+            return super(ObservableMixin, self)._invoke(
+                request, action, stack, suffix)
         return catcher.result
 
     def append_pack(self, pack):
@@ -103,16 +102,17 @@ class ObservableController(ObservableMixin, m3_actions.ActionController):
                         ))
                 return default
 
-    def build_context(self, request, rules):
-        """
-        Выполняет построение контекста вызова операции ActionContext
-        на основе переданного request
-        """
-        if isinstance(rules, dict):
-            from django.conf import settings
-            return self.VerboseDeclarativeContext(debug=settings.DEBUG)
-        else:
-            return m3_actions.ActionContext()
+    # INFO: Всё! Кончилось время боли! Кто не успел, тот опоздал
+    # def build_context(self, request, rules):
+    #     """
+    #     Выполняет построение контекста вызова операции ActionContext
+    #     на основе переданного request
+    #     """
+    #     if isinstance(rules, dict):
+    #         from django.conf import settings
+    #         return self.VerboseDeclarativeContext(debug=settings.DEBUG)
+    #     else:
+    #         return m3_actions.ActionContext()
 
 
 #==============================================================================
@@ -125,7 +125,7 @@ class Observer(object):
     # уровни детализации отладочного логировния
     LOG_NONE, LOG_WARNINGS, LOG_CALLS, LOG_MORE = 0, 1, 2, 3
 
-    class _BeforeAfterPack:
+    class _BeforAfterPack:
         """
         Обертка для списка слушателей, реализующая вызов before/after
         в соответствии с приоритетом, определяемым порякрм слушателей
@@ -188,8 +188,6 @@ class Observer(object):
         self._model_register = {}
         self._pack_instances_by_name = {}
 
-        self._is_configured = False
-
     def get(self, model_name):
         """
         Поиск экземпляра ActionPack для модели по имени её класса.
@@ -233,18 +231,10 @@ class Observer(object):
 
         return name
 
-    def configure(self, force=False):
+    def _reconfigure(self):
         """
-        Построение дерева сопоставления экшнов со слушателями
-        Если observer был сконфигурирован ранее и в него ничего не добавили,
-        то построение выполнится, только если передан аргумент `force=True`
-
-        :param bool force: Форсировать конфигурирование
+        Перестройка дерева сопоставления экшнов со слушателями
         """
-        # конфижим листенеры только понеобходимости
-        if self._is_configured and not force:
-            return
-
         self._action_listeners = {}
         # слушатели сортируются по приоритету
         listeners = [
@@ -263,15 +253,13 @@ class Observer(object):
                         '\n\tListener\t %r' % (name, listener))
             self._action_listeners[name] = action_listeners
 
-        self._is_configured = True
-
     def _populate_pack(self, pack):
         """
         Подписка зарегистрированных слушателей на pack.actions
         """
         # каждый отдельный Pack должен регистрироваться ровно один раз
         # уникльность Pack определяется следующим ключом
-        pack_name = tools._name_class(pack.__class__) 
+        pack_name = pack.get_short_name()
         if pack_name in self._pack_instances_by_name:
             # попытка перерегистрации отмечается предупреждением
             raise RuntimeError(
@@ -313,11 +301,7 @@ class Observer(object):
                     'because this name registered for %r!'
                     % (name, action, self._actions[name]))
             self._actions[name] = action
-
-        for subpack in pack.subpacks:
-            self._populate_pack(subpack)
-
-        self._is_configured = False
+        self._reconfigure()
 
     def subscribe(self, listener):
         """
@@ -342,7 +326,7 @@ class Observer(object):
             self.LOG_MORE,
             'Listener registered:\n\tListener %r' % listener)
 
-        self._is_configured = False
+        self._reconfigure()
 
         return listener
 
@@ -432,7 +416,7 @@ class Observer(object):
         # если подписчики есть, в стек паков добавляется "пак",
         # реализующий подписку на before/after
         if listeners:
-            stack.insert(0, self._BeforeAfterPack(
+            stack.insert(0, self._BeforAfterPack(
                 action, listeners,
                 # инжекция логирования
                 logger=lambda msg: self._log(self.LOG_CALLS, msg)
