@@ -62,15 +62,23 @@ class BaseWindow(ext_windows.ExtWindow):
         .. note::
 
             Параметры могут содержать общие настройки окна (title, width,
-            height, maximized) и флаг режима для чтения (read_only)
+            height, maximized...) и флаг режима для чтения (read_only)
 
         :param params: Словарь с параметрами
         :type params: dict
         """
-        self.title = params.get('title', self.title) or u''
-        self.width = params.get('width', self.width)
-        self.height = params.get('height', self.height)
-        self.maximized = params.get('maximized', self.maximized)
+
+        def apply(attr):
+            if attr in params:
+                setattr(self, attr, params[attr])
+
+        apply('width')
+        apply('height')
+        apply('maximized')
+        apply('maximizable')
+        apply('minimizable')
+        apply('title')
+        self.title = self.title or u''
 
         if params.get('read_only'):
             self.make_read_only()
@@ -105,27 +113,19 @@ class BaseEditWindow(ext_windows.ExtEditWindow, BaseWindow):
     Базовое окно редактирования (с формой и кнопкой сабмита)
     """
 
-    @property
-    def form(self):
-        """
-        Форма окна
-        """
-        # Хак для совместимости с m3
-        return self.__form
-
     def _init_components(self):
         """
         .. seealso::
             :func:`objectpack.ui.BaseWindow._init_components`
         """
         super(BaseEditWindow, self)._init_components()
-        self.__form = ext.ExtForm()
-        self.items.append(self.form)
+
+        self.form = ext.ExtForm()
 
         self.save_btn = ext.ExtButton(
             text=u'Сохранить', handler="submitForm")
         self.cancel_btn = ext.ExtButton(
-            text=u'Отмена', handler="cancelForm")
+            text=u'Отмена', handler="close")
 
         # Кнопка "Отмена" не блокируется в режиме "только для чтения"
         self._mro_exclude_list.append(self.cancel_btn)
@@ -136,7 +136,7 @@ class BaseEditWindow(ext_windows.ExtEditWindow, BaseWindow):
             :func:`objectpack.ui.BaseWindow._do_layout`
         """
         super(BaseEditWindow, self)._do_layout()
-        self.modal = True
+        # self.modal = True
 
         self.buttons.extend([
             self.save_btn,
@@ -176,23 +176,20 @@ class BaseListWindow(BaseWindow):
         .. seealso::
             :func:`objectpack.ui.BaseWindow._init_components`
         """
-        self.grid = ext.ExtObjectGrid()
+        self.grid = ext.ExtObjectGrid(item_id='grid')
         self.close_btn = self.btn_close = ext.ExtButton(
             name='close_btn',
             text=u'Закрыть',
-            handler='function(){Ext.getCmp("%s").close();}' % self.client_id
+            handler='close'
         )
-        self._mro_exclude_list.extend([
-            self.close_btn,
-            self.grid.top_bar.button_refresh
-        ])
+        self._mro_exclude_list.append(self.close_btn)
 
     def _do_layout(self):
         """
         .. seealso::
             :func:`objectpack.ui.BaseWindow._do_layout`
         """
-        self.layout = 'fit'
+        self.layout = self.FIT
         self.items.append(self.grid)
         self.buttons.append(self.btn_close)
 
@@ -205,6 +202,7 @@ class BaseListWindow(BaseWindow):
         """
         super(BaseListWindow, self).set_params(params)
         self.maximizable = self.minimizable = True
+
         assert 'pack' in params, (
             u'Параметры окна должны содержать экземпляр ActionPack'
             u' настраивающего grid!'
@@ -232,6 +230,7 @@ class BaseListWindow(BaseWindow):
             fltr = self.grid_filters[column_name]
         else:
             fltr = {}
+
         fltr[filter_name] = {
             'column_name': column_name,
             'filter_control': filter_control,
@@ -303,6 +302,15 @@ class BaseSelectWindow(BaseListWindow):
     """
     Окно выбора из списка объектов
     """
+
+    _xtype = 'objectpack-select-window'
+
+    js_attrs = BaseListWindow.js_attrs.extend(
+        grid_item_id='gridItemId',
+        column_name_on_select='columnNameOnSelect',
+        multi_select='multiSelect',
+    )
+
     def _init_components(self):
         """
         .. seealso::
@@ -310,8 +318,10 @@ class BaseSelectWindow(BaseListWindow):
         """
         super(BaseSelectWindow, self)._init_components()
         self.select_btn = ext.ExtButton(
-            handler='selectValue', text=u'Выбрать')
+            handler='selectValue',
+            text=u'Выбрать')
         self._mro_exclude_list.append(self.select_btn)
+        self.grid_item_id = self.grid.item_id
 
     def _do_layout(self):
         """
@@ -325,6 +335,7 @@ class BaseSelectWindow(BaseListWindow):
         """
         Включает множественный выбор в гриде
         """
+        self.multi_select = True
         self.grid.sm = ext.ExtGridCheckBoxSelModel()
 
     def set_params(self, params):
@@ -335,20 +346,10 @@ class BaseSelectWindow(BaseListWindow):
         super(BaseSelectWindow, self).set_params(params)
         if params.get('multi_select', False):
             self._enable_multi_select()
-        self.template_globals = 'select-window.js'
+
         self.column_name_on_select = params['column_name_on_select']
-        self.grid.handler_dblclick = 'selectValue'
 
 
-class BaseMultiSelectWindow(BaseSelectWindow):
-    """
-    Окно множественного выбора в ExtMultiSelectWindow
-    """
-    def set_params(self, params):
-        super(BaseMultiSelectWindow, self).set_params(params)
-        self._enable_multi_select()
-        self.template_globals = 'multi-select-window.js'
-
 #==============================================================================
 # ColumnsConstructor
 #==============================================================================
@@ -462,14 +463,15 @@ class ColumnsConstructor(object):
             if is_top_level:
                 if not self._cleaned():
                     return 0  # чистка
-                level = self._normalized_depth()  # нормализация уровней
+                self._normalized_depth()  # нормализация уровней
+                level = 1
             else:
                 grid.add_banded_column(self._column, level, 0)
 
             if not self.items:
                 return 0
 
-            cnt = sum([i._populate(grid, level - 1) for i in self.items])
+            cnt = sum([i._populate(grid, level + 1) for i in self.items])
             self._column.colspan = cnt
 
             return cnt
@@ -566,7 +568,7 @@ class ModelEditWindow(BaseEditWindow):
 
     model = None
     """
-    Модель, для которой будет строится окно
+    Модель, для которой будет строиться окно
     """
 
     field_fabric_params = None
@@ -591,8 +593,8 @@ class ModelEditWindow(BaseEditWindow):
         super(ModelEditWindow, self)._do_layout()
 
         # автоматически вычисляемая высота окна
-        self.height = None
-        self.layout = 'form'
+        #self.height = None
+        self.layout = self.FORM
         self.layout_config = {'autoHeight': True}
         self.form.layout_config = {'autoHeight': True}
 
@@ -741,10 +743,6 @@ def _create_control_for_field(f, model_register=None, **kwargs):
 
     elif isinstance(f, django_models.IntegerField):
         ctl = ext.ExtNumberField(**kwargs)
-        ctl.allow_decimals = False
-        if isinstance(f, (django_models.PositiveIntegerField,
-                          django_models.PositiveSmallIntegerField)):
-            ctl.allow_negative = False
 
     elif isinstance(f, django_models.FloatField):
         ctl = ext.ExtNumberField(**kwargs)
@@ -897,13 +895,13 @@ class TabbedWindow(BaseWindow):
 
         # инстанцирование вкладок
         instantiate = lambda x: x() if inspect.isclass(x) else x
-        self.tabs = map(instantiate, self.tabs or [])
+        self._tabs = map(instantiate, self.tabs or [])
 
         # опредение вкладок не должно быть пустым
         # (проверка производится после инстанцирования,
         # т.к. описание колонок может быть итератором
         # и иметь истинное значение в булевом контексте)
-        assert self.tabs, '"tabs" can not be empty!'
+        assert self._tabs, '"tabs" can not be empty!'
 
         super(TabbedWindow, self)._init_components()
 
@@ -911,21 +909,21 @@ class TabbedWindow(BaseWindow):
         self._tab_container = ext.ExtTabPanel(deferred_render=False)
 
         # создание компонентов для вкладок
-        for con in self.tabs:
+        for con in self._tabs:
             con.init_components(win=self)
 
     def _do_layout(self):
         super(TabbedWindow, self)._do_layout()
 
         # настройка отображения окна
-        self.layout = 'fit'
+        self.layout = self.FIT
         self.width, self.height = 600, 450
         self.min_width, self.min_height = self.width, self.height
 
         self.maximizable = self.minimizable = True
 
         # размещение контролов во вкладках
-        for con in self.tabs:
+        for con in self._tabs:
             tab = con._create_tab()
             con.do_layout(win=self, tab=tab)
             self._tab_container.items.append(tab)
@@ -933,7 +931,7 @@ class TabbedWindow(BaseWindow):
         # размещение контейнера вкладок на форму
         tc = self._tab_container
         tc.anchor = '100%'
-        tc.layout = 'fit'
+        # tc.layout = self.FIT
         tc.auto_scroll = True
         self.items.append(tc)
 
@@ -942,7 +940,7 @@ class TabbedWindow(BaseWindow):
 
         # установка параметров вкладок, формирование списка шаблонов вкладок
         self.tabs_templates = []
-        for con in self.tabs:
+        for con in self._tabs:
             if con.template:
                 self.tabs_templates.append(con.template)
             con.set_params(win=self, params=params)
@@ -969,7 +967,10 @@ class TabbedEditWindow(TabbedWindow, BaseEditWindow):
         super(TabbedEditWindow, self)._do_layout()
         self.items.remove(self._tab_container)
         self.form.items.append(self._tab_container)
-        self.form.layout = 'fit'
+        self.form.layout = self.form.FIT
+
+        self.form.padding = None
+        self._tab_container.border = False
 
 
 #==============================================================================
@@ -1002,7 +1003,8 @@ class ObjectGridTab(WindowTab):
 
     def _create_tab(self):
         tab = super(ObjectGridTab, self)._create_tab()
-        tab.layout = 'fit'
+        tab.padding = None
+        tab.layout = tab.FIT
         return tab
 
     def init_components(self, win):
@@ -1073,15 +1075,11 @@ class ObjectTab(WindowTab):
     """
     Вкладка редактирования полей объекта
     """
+    # Модель, для которой будет строится окно
     model = None
-    """
-    Модель, для которой будет строится окно
-    """
 
+    # Словарь kwargs для model_fields_to_controls ("field_list", и т.д.)
     field_fabric_params = None
-    """
-    Словарь kwargs для model_fields_to_controls ("field_list", и т.д.)
-    """
 
     @property
     def title(self):
@@ -1109,13 +1107,14 @@ class ObjectTab(WindowTab):
         # все поля добавляются на форму растянутыми по ширине
         tab.items.extend(map(anchor100, self._controls))
 
-    def set_params(self, win, params):
-        super(ObjectTab, self).set_params(win, params)
-        # если сгенерировано хотя бы одно поле загрузки файлов,
-        # окно получает флаг разрешения загрузки файлов
-        win.form.file_upload = win.form.file_upload or any(
-            isinstance(x, ext.ExtFileUploadField)
-            for x in self._controls)
+    # TODO: хз, м.б. теперь file_upload и не нужен
+    # def set_params(self, win, params):
+    #     super(ObjectTab, self).set_params(win, params)
+    #     # если сгенерировано хотя бы одно поле загрузки файлов,
+    #     # окно получает флаг разрешения загрузки файлов
+    #     win.form.file_upload = win.form.file_upload or any(
+    #         isinstance(x, ext.ExtFileUploadField)
+    #         for x in self._controls)
 
     @classmethod
     def fabricate(cls, model, **kwargs):
@@ -1169,6 +1168,7 @@ class ComboBoxWithStore(ext.ExtDictSelectField):
             self.store.root = 'rows'
         else:
             self.store = ext_store.ExtDataStore(data or ((0, ''),))
+            self.mode = self.store.LOCAL
 
     @property
     def data(self):
@@ -1244,3 +1244,36 @@ deny_blank.__doc__ = """
         controls = map(allow_blank, controls)
 
     """
+
+#==============================================================================
+# BaseMasterDetailWindow
+#==============================================================================
+class MasterDetailWindow(BaseWindow):
+    """
+    Основа для окна master-detail
+    """
+    _xtype = 'objectpack-master-detail-window'
+
+    master_grid_clz = ext.ExtObjectGrid
+    detail_grid_clz = ext.ExtObjectGrid
+
+    def _init_components(self):
+        self.master_grid = self.master_grid_clz()
+        self.detail_grid = self.detail_grid_clz()
+
+    def _do_layout(self):
+        self.layout = 'border'
+        self.master_grid.region = 'west'
+        self.master_grid.width = 300
+        self.detail_grid.region = 'center'
+        self.items.extend([
+            self.master_grid,
+            self.detail_grid
+        ])
+
+    def set_params(self, params):
+        super(MasterDetailWindow, self).set_params(params)
+        params['master_pack'].configure_grid(self.master_grid)
+        params['detail_pack'].configure_grid(self.detail_grid)
+        self.master_param_name = params['master_pack'].id_param_name
+        self.choose_master_msg = params.get('choose_master_msg')
